package com.aizuda.boot.modules.flw.flow;

import com.aizuda.boot.modules.system.service.ISysUserDepartmentService;
import com.aizuda.boot.modules.system.service.ISysUserRoleService;
import com.aizuda.bpm.engine.TaskAccessStrategy;
import com.aizuda.bpm.engine.assist.ObjectUtils;
import com.aizuda.bpm.engine.entity.FlwTaskActor;
import com.aizuda.service.spring.SpringHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Objects;

@Component
public class FlowTaskAccessStrategy implements TaskAccessStrategy {

    @Override
    public FlwTaskActor isAllowed(String userId, List<FlwTaskActor> taskActors) {
        if (null == userId || ObjectUtils.isEmpty(taskActors)) {
            return null;
        }
        FlwTaskActor flwTaskActor = taskActors.get(0);
        if (Objects.equals(flwTaskActor.getActorType(), 1)) {
            // 角色
            ISysUserRoleService sysUserRoleService = SpringHelper.getBean(ISysUserRoleService.class);
            return contains(taskActors, sysUserRoleService.listRoleIdsByUserId(Long.valueOf(userId)));
        } else if (Objects.equals(flwTaskActor.getActorType(), 2)) {
            // 部门
            ISysUserDepartmentService sysUserDepartmentService = SpringHelper.getBean(ISysUserDepartmentService.class);
            return contains(taskActors, sysUserDepartmentService.listDepartmentIdsByUserId(Long.valueOf(userId)));
        }

        // 参与者 ID 默认非组，作为用户ID判断是否允许执行
        return taskActors.stream().filter(t -> Objects.equals(t.getActorId(), userId)).findFirst().orElse(null);
    }

    private FlwTaskActor contains(List<FlwTaskActor> taskActors, List<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return null;
        }
        return taskActors.stream().filter(t -> ids.contains(Long.valueOf(t.getActorId()))).findFirst().orElse(null);
    }
}
